const St = imports.gi.St;
const Main = imports.ui.main;
const Mainloop = imports.mainloop;
const Lang = imports.lang;
const GLib = imports.gi.GLib;
const Gio = imports.gi.Gio;
const Shell = imports.gi.Shell;

const timerVal = 2000;
const unitMem = 'Go'; //Mo ou Go

let cpuMemInfo;
let meta;

function CpuMemInfo()
{   
    CpuMemInfo.prototype.construct = function()
    {
        this.icon = new St.Icon(
        {
//	    gicon: Gio.icon_new_for_string(meta.path + "/processor.png"),
	    icon_name: "utilities-system-monitor-symbolic",
	    icon_type: St.IconType.SYMBOLIC,
	    icon_size: 16,
	    style_class: 'cmi-icon'
        });

	this.labelCpu = new St.Label({
	    style_class: 'cmi-cpu-label'
	});

	this.unitCpu = new St.Label({
	    text: '%',
	    style_class: 'cmi-cpu-unit-label'
	});

	this.labelMem = new St.Label({
	    style_class: 'cmi-mem-label'
	});

	this.unitMem = new St.Label({
	    text: unitMem,
	    style_class: 'cmi-mem-unit-label'
	});

	this.box = new St.BoxLayout();
        this.box.add_actor(this.icon);
        this.box.add_actor(this.labelCpu);
        this.box.add_actor(this.unitCpu);
        this.box.add_actor(this.labelMem);
        this.box.add_actor(this.unitMem);

	this.button = new St.Button({
	    style_class: 'panel-button'
	});
        this.button.connect('clicked', this.clicked);
        this.button.set_child(this.box);

	Main.panel._rightBox.insert_child_at_index(this.button, 0);

	this.timer = Mainloop.timeout_add(timerVal, Lang.bind(this, this.update));
	this.update();
    }

    CpuMemInfo.prototype.destruct = function()
    {
	Main.panel._rightBox.remove_child(this.button);

        Mainloop.source_remove(this.timer);
    }

    CpuMemInfo.prototype.clicked = function()
    {
	Shell.AppSystem.get_default().lookup_app('gnome-system-monitor.desktop').activate();	
    }

    CpuMemInfo.prototype.update = function()
    {
	let cpuTotal = 0, cpuIdle;

	let fileStat = GLib.file_get_contents('/proc/stat');
        linesStat = ("" + fileStat[1]).split("\n");

        for (let i = 0; i < linesStat.length; i++)
	{
            let values;

            if (linesStat[i].match(/^cpu /))
            {
                values = linesStat[i].match(/^cpu\s*(.*)$/)[1].split(' ');

		cpuIdle = values[3];

		for (let i = 0; i < values.length; i++)
		    cpuTotal += parseInt(values[i]);
            }
	}

	if (typeof(this.cpuTotalOld) == "undefined" && typeof(this.cpuIdleOld) == "undefined")
	{
	    this.cpuTotalOld = cpuTotal - 1;
	    this.cpuIdleOld = cpuIdle - 1;
	}

	let diffCpuTotal = cpuTotal - this.cpuTotalOld;
	let diffCpuIdle = cpuIdle - this.cpuIdleOld;

	let cpuValue = Math.round(100 * (diffCpuTotal - diffCpuIdle) / diffCpuTotal);

	this.cpuTotalOld = cpuTotal;
	this.cpuIdleOld = cpuIdle;

        this.labelCpu.set_text(cpuValue.toString());



	let memTotal, memFree, memBuffers, memCached;

	let fileMeminfo = GLib.file_get_contents('/proc/meminfo');
	linesMeminfo = ("" + fileMeminfo[1]).split("\n");

	for (let i = 0; i < linesMeminfo.length; i++)
	{
	    let values;

	    if (linesMeminfo[i].match(/^MemTotal/))
	    {
		values = linesMeminfo[i].match(/^MemTotal:\s*([^ ]*)\s*([^ ]*)$/);
		
		memTotal = values[1];
	    }
	    else if (linesMeminfo[i].match(/^MemFree/))
	    {
		values = linesMeminfo[i].match(/^MemFree:\s*([^ ]*)\s*([^ ]*)$/);
		
		memFree = values[1];
	    }
	    else if (linesMeminfo[i].match(/^Buffers/))
	    {
		values = linesMeminfo[i].match(/^Buffers:\s*([^ ]*)\s*([^ ]*)$/);
		
		memBuffers = values[1];
	    }
	    else if (linesMeminfo[i].match(/^Cached/))
	    {
		values = linesMeminfo[i].match(/^Cached:\s*([^ ]*)\s*([^ ]*)$/);
		
		memCached = values[1];
	    }	    
        }

	let memUsed = memTotal - memFree - memBuffers - memCached;

	let precision = 1;
	let unitFactor = 1024;

	if (unitMem == "Go")
	{
	    unitFactor = 1048576;

	    if (memTotal >= 10 * unitFactor)
		precision = 10;
	    else
		precision = 100;
	}

	memUsed = Math.round(precision * memUsed / unitFactor) / precision;
	memTotal = Math.round(precision * memTotal / unitFactor) / precision;

        this.labelMem.set_text(memUsed.toString() + " / " + memTotal.toString());

	return true;
    }

    this.construct();
}

function init(extensionMeta)
{
    meta = extensionMeta;
}

function enable()
{
    cpuMemInfo = new CpuMemInfo();
}

function disable()
{
    cpuMemInfo.destruct();
}
